const path = require('path')
const fs = require('fs')
const cssLoaders = require('./css-loaders')
const MpvuePlugin = require('webpack-mpvue-asset-plugin')
const UploadPlugin = require('./upload')
const ExtractTextPlugin = require('extract-text-webpack-plugin')
const CopyWebpackPlugin = require('copy-webpack-plugin')
const webpack = require('webpack')
const CleanWebpackPlugin = require('clean-webpack-plugin')
const paths = require('./paths')

const appDirectory = process.cwd()
function resolve(dir) {
  return path.join(appDirectory, dir)
}

const defaultEntry = `import Vue from 'vue'
import App from './index'

const app = new Vue(App)
app.$mount()
`

function genPageEntries() {
  const pagePath = path.join(resolve('./src'), 'pages')
  const dirs = fs
    .readdirSync(pagePath)
    .filter(f => fs.statSync(path.join(pagePath, f)).isDirectory())
    .map(dir => path.join(pagePath, dir))

  const pages = []
  const entries = dirs.reduce((res, dir) => {
    const entryFile = path.join(dir, 'main.js')
    if (!fs.existsSync(entryFile)) {
      fs.writeFileSync(entryFile, defaultEntry)
    }
    // 转换入口文件名到小程序配置文件的page名称
    // src/pages/index/main.js -> pages/index/main
    const entryName = entryFile
      .split('/')
      .slice(-3)
      .join('/')
      .slice(0, -3)
    res[entryName] = entryFile
    pages.push(entryName)

    return res
  }, {})
  const appJsonPath = `${appDirectory}/src/app.json`
  const appJson = require(appJsonPath)
  function pagesEqual(a, b) {
    if (a.length !== b.length) return false
    for (let i of a) if (!b.includes(i)) return false
    return true
  }
  if (!pagesEqual(appJson.pages, pages)) {
    const indexPage = 'pages/index/main'
    const indexPositon = pages.findIndex(x => x === indexPage)
    if (indexPositon === -1) {
      throw 'index页面没有配置'
    }
    pages[indexPositon] = pages[0]
    pages[0] = indexPage
    appJson.pages = pages
    fs.writeFileSync(resolve('./src/app.json'), JSON.stringify(appJson))
  }
  return entries
}

process.env.NODE_ENV = 'production'

module.exports = {
  entry: {
    app: resolve('./src/main.js'),
    ...genPageEntries(),
  },
  target: require('mpvue-webpack-target'),
  output: {
    publicPath: process.env.PUBLIC_PATH || 'https://wxappcdn.zanquan.net/',
    path: path.resolve('./dist'),
    filename: '[name].js',
    chunkFilename: '[id].js',
  },
  resolve: {
    modules: [path.resolve(paths.appPath, 'src'), 'node_modules'],
    extensions: ['.js', '.vue', '.json'],
    alias: {
      vue: 'mpvue',
      '@': resolve('src'),
    },
    symlinks: true,
    aliasFields: ['mpvue', 'weapp', 'browser'],
    mainFields: ['browser', 'module', 'main'],
  },
  module: {
    rules: [
      ...cssLoaders.loaders(),
      {
        test: /\.(graphql|gql)$/,
        exclude: /node_modules/,
        loader: 'graphql-tag/loader',
      },
      {
        test: /\.vue$/,
        include: paths.srcPaths,
        loader: 'soogang-mpvue-loader',
        options: { loaders: cssLoaders.vueLoaders() },
      },
      {
        test: /\.js$/,
        include: paths.srcPaths,
        use: [
          'babel-loader',
          {
            loader: 'soogang-mpvue-loader',
            options: {
              checkMPEntry: true,
            },
          },
        ],
      },
      {
        test: /\.(png|jpe?g|gif|svg)(\?.*)?$/,
        loader: 'url-loader',
        options: {
          limit: 10000,
          name: 'asset/img/[name].[hash].[ext]',
        },
      },
      {
        test: /\.(mp4|webm|ogg|mp3|wav|flac|aac)(\?.*)?$/,
        loader: 'url-loader',
        options: {
          limit: 10000,
          name: 'asset/media/[name].[hash].[ext]',
        },
      },
      {
        test: /\.(woff2?|eot|ttf|otf)(\?.*)?$/,
        loader: 'url-loader',
        options: {
          limit: 10000,
          name: 'asset/fonts/[name].[hash].[ext]',
        },
      },
    ],
  },
  plugins: [
    new CleanWebpackPlugin(resolve('./dist'), {
      root: resolve('./'),
    }),
    new webpack.DefinePlugin({
      'process.env.NODE_ENV': JSON.stringify(process.env.NODE_ENV),
    }),
    new webpack.optimize.CommonsChunkPlugin({
      name: 'vendor',
      minChunks: function(module, count) {
        return (
          (module.resource &&
            /\.js$/.test(module.resource) &&
            module.resource.indexOf('node_modules') >= 0) ||
          count > 1
        )
      },
    }),
    new webpack.optimize.CommonsChunkPlugin({
      name: 'manifest',
      minChunks: Infinity,
    }),
    new MpvuePlugin(),
    new CopyWebpackPlugin(
      [
        {
          from: '**/*.json',
          to: '',
        },
      ],
      {
        context: 'src/',
      }
    ),
    // 推荐通过 require 使用图片
    fs.existsSync('./static')
      ? new CopyWebpackPlugin([
          {
            from: resolve('./static'),
            to: resolve('./dist/static'),
            ignore: ['.*'],
          },
        ])
      : null,
    new ExtractTextPlugin({
      filename: '[name].wxss',
    }),

    new UploadPlugin(), // 上传图片到七牛cdn
  ].filter(Boolean),
}
